import { ActorSystem } from "./ActorSystem";
import { Address, Handler, Strategy } from "./interfaces";
export declare type MailBoxMessage<T> = {
    type: ValidActorMethodPropNames<T>;
    payload: PayloadPropNames<T>[];
    senderAddress: Address | null;
    callback: (error?: any, result?: any) => void;
};
export declare type ValidActorMethodProps<T> = Pick<T, ValidActorMethodPropNames<T>>;
export declare type ValidActorMethodPropNames<T> = {
    [K in Exclude<keyof T, keyof Actor>]: T[K] extends (...args: any[]) => infer R ? R extends Promise<any> ? K : never : never;
}[Exclude<keyof T, keyof Actor>];
export declare type PayloadPropNames<T> = {
    [K in Exclude<keyof T, keyof Actor>]: T[K] extends (_: infer S) => Promise<any> ? S : never;
}[Exclude<keyof T, keyof Actor>];
export declare type ActorCons<T extends Actor<K>, K = undefined> = new (name: string, address: Address, actorSystem: ActorSystem, options?: K, strategies?: Strategy[]) => T;
export declare class ActorRef<T> {
    address: Address;
    private actorSystem;
    constructor(address: Address, actorSystem: ActorSystem);
    invoke(sender?: Address): Pick<T, { [K in Exclude<keyof T, "at" | "atSelf" | "onNewMessage" | "pushToMailbox" | "ref">]: T[K] extends (...args: any[]) => infer R ? R extends Promise<any> ? K : never : never; }[Exclude<keyof T, "at" | "atSelf" | "onNewMessage" | "pushToMailbox" | "ref">]>;
}
export declare abstract class Actor<InitParam = undefined> {
    protected address: Address;
    protected actorSystem: ActorSystem;
    protected strategies: "IgnoreOlderMessageWithTheSameType"[] | undefined;
    protected name: string;
    protected mailBox: MailBoxMessage<this>[];
    protected currentlyProcessedMessage: MailBoxMessage<this> | undefined;
    protected context: {
        senderAddress: Address | null;
        senderRef: ActorRef<any> | null;
    };
    private timerId;
    private currentPromise;
    constructor(name: string, address: Address, actorSystem: ActorSystem, options?: InitParam, strategies?: "IgnoreOlderMessageWithTheSameType"[] | undefined);
    at<A>(targetRef: ActorRef<A> | Address): Handler<A>;
    atSelf(): Handler<any>;
    onNewMessage: <K extends { [K in Exclude<keyof this, "at" | "atSelf" | "onNewMessage" | "pushToMailbox" | "ref">]: this[K] extends (...args: any[]) => infer R ? R extends Promise<any> ? K : never : never; }[Exclude<keyof this, "at" | "atSelf" | "onNewMessage" | "pushToMailbox" | "ref">], L extends { [K in Exclude<keyof this, "at" | "atSelf" | "onNewMessage" | "pushToMailbox" | "ref">]: this[K] extends (_: infer S) => Promise<any> ? S : never; }[Exclude<keyof this, "at" | "atSelf" | "onNewMessage" | "pushToMailbox" | "ref">]>(type: K, senderAddress: Address | null, ...payload: L[]) => void;
    pushToMailbox: <K extends { [K in Exclude<keyof this, "at" | "atSelf" | "onNewMessage" | "pushToMailbox" | "ref">]: this[K] extends (...args: any[]) => infer R ? R extends Promise<any> ? K : never : never; }[Exclude<keyof this, "at" | "atSelf" | "onNewMessage" | "pushToMailbox" | "ref">], L extends { [K in Exclude<keyof this, "at" | "atSelf" | "onNewMessage" | "pushToMailbox" | "ref">]: this[K] extends (_: infer S) => Promise<any> ? S : never; }[Exclude<keyof this, "at" | "atSelf" | "onNewMessage" | "pushToMailbox" | "ref">]>(type: K, senderAddress: Address | null, ...payload: L[]) => Promise<any>;
    ref: <T>(address: Address) => ActorRef<T>;
    protected init(options?: InitParam): void;
    protected log(...message: any[]): void;
    protected cancelCurrentExecution: () => void;
    private handleMessage<K>(type, ...payload);
    private scheduleNextTick;
    private executeTick;
}
